# -*- coding: utf-8 -*-
"""
    clihub/gist.py
    ~~~~~~~~~~~~~~

    :copyright: (c) 2012 by Stephane Wirtel <stephane@wirtel.be>
    :license: BSD, see LICENSE for more details
"""
import requests
import json


class Gist(object):
    @classmethod
    def create(cls, account, files, description=None, private=False):
        hash_files = {}
        for filename in files:
            with open(filename, 'r') as fp:
                hash_files[filename] = {
                    'content': fp.read(),
                }

        values = {
            'description': description or 'Undefined Description',
            'public': not private,
            'files': hash_files
        }
        url = 'https://api.github.com/gists'

        query = requests.post(url,
                              auth=(account.username, account.password),
                              data=json.dumps(values))

        if query.status_code == 201:
            return query.json()['html_url']

    @classmethod
    def delete(cls, account, gist_id):
        url = 'https://api.github.com/gists/{gist_id}'
        url = url.format(gist_id=gist_id)

        requests.delete(url, auth=(account.username, account.password))

    def __init__(self, account):
        self.account = account



# -*- coding: utf-8 -*-
import os
import imp
from contextlib import contextmanager


class Account(object):
    def __init__(self, username, password):
        self.username = username
        self.password = password

    @classmethod
    @contextmanager
    def load_from_config(cls):
        config_file = cls.config_file()
        d = imp.new_module('config')
        d.__file__ = config_file
        try:
            execfile(config_file, d.__dict__)
            yield Account(d.username, d.password)
        except IOError, e:
            e.strerror = 'Unable to load configuration file (%s)' % e.strerror
            raise

    @classmethod
    def save_to_config(cls, username, password):
        try:
            with open(cls.config_file(), 'w') as fp:
                fp.write('username="{username}"\npassword="{password}"\n'.format(username=username, password=password))
        except IOError, e:
            e.strerror = 'Unable to save configuration file (%s)' % e.strerror
            raise

    @classmethod
    def remove_config(cls):
        try:
            os.unlink(cls.config_file())
        except OSError:
            pass

    @classmethod
    def config_file(cls):
        return os.path.join(os.environ['HOME'], '.clihub')



# -*- coding: utf-8 -*-
"""
    clihub/repository.py
    ~~~~~~~~~~~~~~~~~~~~

    :copyright: (c) 2012 by Stephane Wirtel <stephane@wirtel.be>
    :license: BSD, see LICENSE for more details
"""
import requests
import json


class Repository(object):
    @classmethod
    def create(cls, account, repository_name, has_issues=True, has_wiki=True):
        values = {
            "name": repository_name,
            "private": False,
            "has_issues": has_issues,
            "has_wiki": has_wiki,
            "has_downloads": False      # XXX deprecated: https://github.com/blog/1302-goodbye-uploads
        }
        url = 'https://api.github.com/user/repos'

        query = requests.post(url,
                              auth=(account.username, account.password),
                              data=json.dumps(values))

        if query.status_code == 201:
            print query.json()['ssh_url']

            return Repository(account, repository_name)
        else:
            raise Exception("Can't create the repository")

    def __init__(self, account, repository_name):
        self.account = account
        self.repository_name = repository_name

    def delete(self):
        url = 'https://api.github.com/repos/{username}/{repository}'
        url = url.format(username=self.account.username,
                         repository=self.repository_name)

        requests.delete(url,
                        auth=(self.account.username, self.account.password))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Usage:
    clihub login <username> <password>
    clihub logout
    clihub repo new <name> [--no-wiki] [--no-issues]
    clihub repo delete <name>
    clihub gist new [--description=DESCRIPTION] [--private] FILES...
    clihub gist delete <name>
    clihub issue new <repository> <title> [--description=DESCRIPTION]

Options:
    -h, --help      Show this screen
    --version       Show version

"""
from docopt import docopt
from ..account import Account
from ..repository import Repository
from ..gist import Gist
from ..issue import Issue
from ..release import version


def main():
    arguments = docopt(__doc__, version=version)
    if arguments.get('login'):
        return Account.save_to_config(arguments.get('<username>'),
                                      arguments.get('<password>'))

    if arguments.get('logout'):
        return Account.remove_config()

    if arguments.get('repo'):
        with Account.load_from_config() as account:
            if arguments.get('new'):
                has_issues = not arguments.get('--no-issues')
                has_wiki = not arguments.get('--no-wiki')
                Repository.create(account, arguments.get('<name>'),
                                  has_issues=has_issues,
                                  has_wiki=has_wiki)
            if arguments.get('delete'):
                Repository(account, arguments.get('<name>')).delete()

    if arguments.get('gist'):
        with Account.load_from_config() as account:
            if arguments.get('new'):
                description = arguments.get('--description') or 'Undefined Description'
                private = arguments.get('--private') or False
                gist = Gist.create(account, arguments.get('FILES'),
                                   description=description,
                                   private=private)
                print gist
            if arguments.get('delete'):
                Gist.delete(account, arguments.get('<name>'))

    if arguments.get('issue'):
        with Account.load_from_config() as account:
            if arguments.get('new'):
                Issue.create(account, arguments.get('<repository>'), arguments.get('<title>'), arguments.get('--description'))

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
clihub
------

clihub is a tool to interact with the APIv3 of GitHUB.

"""
from setuptools import setup
from setuptools import find_packages
from clihub import release


setup(
    name=release.name,
    version=release.version,
    url='http://github.com/matrixise/clihub',
    author='Stephane Wirtel',
    author_email='stephane@wirtel.be',
    install_requires=[
        'docopt',
        'requests',
    ],
    packages=find_packages(),
    platforms='any',
    zip_safe=False,
    entry_points="""
    [console_scripts]
    clihub = clihub.scripts.clihub:main
    """
)



# -*- coding: utf-8 -*-
"""
    clihub/issue.py
    ~~~~~~~~~~~~~~~

    :copyright: (c) 2012 by Stephane Wirtel <stephane@wirtel.be>
    :license: BSD, see LICENSE for more details
"""
import requests
import json


class Issue(object):
    @classmethod
    def create(cls, account, repository_name, title, body):
        values = {
            'title': title,
            'body': body,
        }
        url = 'https://api.github.com/repos/{username}/{repository}/issues'.format(username=account.username,
                                                                                   repository=repository_name)

        query = requests.post(url,
                              auth=(account.username, account.password),
                              data=json.dumps(values))

        if query.status_code == 201:
            print query.json()['url']
